import { system, CommandPermissionLevel, CustomCommandStatus } from "@minecraft/server";
import { ModalFormData } from "@minecraft/server-ui";
system.beforeEvents.startup.subscribe((data) => {
    const configurationCommand = {
        name: "waio:configuration",
        description: "waio.command.configuration",
        permissionLevel: CommandPermissionLevel.Any
    };
    data.customCommandRegistry.registerCommand(configurationCommand, configurationCustomCommand);
});
function configurationCustomCommand(origin) {
    const player = origin.sourceEntity;
    const waio = player.getDynamicProperty("waio");
    const noLang = player.getDynamicProperty("noLang");
    const maxDistance = player.getDynamicProperty("maxDistance");
    const maxItemsDisplay = player.getDynamicProperty("maxItemsDisplay");
    const developer = player.getDynamicProperty("developer");
    const configurationForm = new ModalFormData();
    configurationForm.title("WAIO Configuration");
    configurationForm.label("What Am I Observing?");
    configurationForm.toggle({ translate: "waio.ui.active" }, { defaultValue: waio });
    configurationForm.divider();
    configurationForm.label({ translate: "waio.ui.label.personal_preferences" });
    configurationForm.toggle({ translate: "waio.ui.no_lang" }, { defaultValue: noLang, tooltip: "If disabled, the addon identifier will be formatted and used as addon name." });
    configurationForm.slider({ translate: "waio.ui.max_distance" }, 1, 15, { defaultValue: maxDistance, tooltip: "Modifies the maximum distance (in blocks) at which you will get information about what you are observing." });
    configurationForm.slider({ translate: "waio.ui.max_items_display" }, 1, 10, { defaultValue: maxItemsDisplay, tooltip: "Modifies the maximum number of items that the containers will display (if the value is higher, it is usually more uncomfortable)." });
    configurationForm.divider();
    configurationForm.label({ translate: "waio.ui.label.developer_mode" });
    configurationForm.toggle({ translate: "waio.ui.active" }, { defaultValue: developer, tooltip: "If enabled, it will allow you to see extra information, usually only useful for developers." });
    system.runTimeout(() => {
        configurationForm.show(player).then(async ({ canceled, formValues }) => {
            if (canceled)
                return;
            player.setDynamicProperty("waio", formValues[1]);
            player.setDynamicProperty("noLang", formValues[4]);
            player.setDynamicProperty("maxDistance", formValues[5]);
            player.setDynamicProperty("maxItemsDisplay", formValues[6]);
            player.setDynamicProperty("developer", formValues[9]);
            player.sendMessage("§aConfiguration has been successfully updated.");
        });
    });
    return {
        status: CustomCommandStatus.Success
    };
}
