import { BlockStates } from "@minecraft/server";
export function getPack(id, noLang) {
    let pack = "Minecraft";
    if (id !== "minecraft")
        pack = noLang ? id.split("_").map((word) => word.charAt(0).toUpperCase() + word.slice(1)).join(" ") : `waio.${id}:description.name`;
    return pack;
}
export function getHealth(currentHealth, maxHealth) {
    const icons = {
        fullHeart: "",
        halfHeart: "",
        emptyHeart: ""
    };
    if (maxHealth <= 40) {
        const totalHearts = Math.ceil(maxHealth / 2);
        const fullHearts = Math.floor(currentHealth / 2);
        const halfHeart = currentHealth % 2 === 1 && fullHearts < totalHearts;
        const emptyHearts = totalHearts - fullHearts - (halfHeart ? 1 : 0);
        let showHearts = `${icons.fullHeart.repeat(fullHearts)}${halfHeart ? icons.halfHeart : ""}${icons.emptyHeart.repeat(emptyHearts)}`;
        return showHearts.length <= 10
            ? showHearts
            : `${showHearts.substring(0, 10)}\n${showHearts.substring(10)}`;
    }
    else {
        return `${icons.fullHeart} §7${currentHealth}/${maxHealth}`;
    }
}
export function time(tick) {
    const min = Math.floor(tick / 1200);
    const sec = Math.floor(tick / 20) - min * 60;
    if (tick === -1)
        return "Infinite";
    return `${min}:${sec.toString().length === 1 ? `0${sec}` : sec}`;
}
export function getItems(block, showLimit = 6, itemsPerLine = 3) {
    const inv = block.getComponent("inventory")?.container;
    if (!inv)
        return [];
    const translate = [];
    const foundItems = [];
    for (let i = 0; i < inv.size; i++) {
        const item = inv.getItem(i);
        if (item) {
            foundItems.push(item);
        }
    }
    const moreItems = foundItems.length > showLimit;
    const itemsToShow = Math.min(showLimit, foundItems.length);
    for (let i = 0; i < itemsToShow; i++) {
        const item = foundItems[i];
        translate.push({ translate: item.localizationKey });
        const isLastDisplayed = i === itemsToShow - 1;
        if (isLastDisplayed) {
            if (moreItems) {
                translate.push({ text: "..." });
            }
            else {
                translate.push({ text: "." });
            }
        }
        else {
            translate.push({ text: ", " });
            if ((i + 1) % itemsPerLine === 0 && i + 1 < itemsToShow) {
                translate.push({ text: "\n" });
            }
        }
    }
    return translate;
}
export function getValidValues(block, keysToCheck) {
    const allValues = {};
    keysToCheck.forEach((key) => {
        const validValues = BlockStates.get(key)?.validValues;
        if (validValues) {
            allValues[key] = validValues;
        }
    });
    for (const key of keysToCheck) {
        const currentValue = block.permutation.getState(key);
        const values = allValues[key];
        if (currentValue !== undefined && values) {
            return { current: currentValue, last: values[values.length - 1] };
        }
    }
    return undefined;
}
export function getAllValues(states, block, text) {
    const allValues = {};
    Object.keys(states).forEach((stateName) => {
        const validValues = BlockStates.get(stateName).validValues;
        const currentState = block.permutation.getState(stateName);
        const updatedValidValues = validValues.map(value => {
            return value === currentState ? `§c${value}§7` : value;
        });
        allValues[stateName] = updatedValidValues;
    });
    let allValuesString = JSON.stringify(allValues);
    allValuesString = allValuesString.replace(/"([^"]+)":/g, '\n"$1":');
    allValuesString = allValuesString.replace(/\[/g, ' [');
    allValuesString = allValuesString.replace(/,/g, ', ');
    allValuesString = allValuesString.slice(1, -1);
    allValuesString = allValuesString.replace(/"(§?[^"]+)"/g, '$1');
    text += `\n\n§7All States: §7${allValuesString}`;
    return text;
}
export function getGrowthPercentage(growth) {
    const current = growth.current;
    const last = growth.last;
    const result = Math.abs(current) / Math.abs(last);
    const rounded = result !== 0 ? result.toFixed(2) : 0;
    const growthText = rounded === "1.00" ? "§aMature" : `${rounded}%`;
    return growthText;
}
export function getEffectColor(id) {
    const badEffects = [
        "minecraft:instant_damage",
        "minecraft:poison",
        "minecraft:weakness",
        "minecraft:slowness",
        "minecraft:wither",
        "minecraft:wind_charged",
        "minecraft:weaving",
        "minecraft:oozing",
        "minecraft:infested",
    ];
    if (badEffects.includes(id))
        return "§c";
    return "§a";
}
const blocks = {
    //! Wool
    "minecraft:white_wool": {
        lang: "tile.wool.white.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:light_gray_wool": {
        lang: "tile.wool.Silver.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:gray_wool": {
        lang: "tile.wool.Gray.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:black_wool": {
        lang: "tile.wool.Black.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:brown_wool": {
        lang: "tile.wool.Brown.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:red_wool": {
        lang: "tile.wool.Red.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:orange_wool": {
        lang: "tile.wool.Orange.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:yellow_wool": {
        lang: "tile.wool.Yellow.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:lime_wool": {
        lang: "tile.wool.Lime.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:green_wool": {
        lang: "tile.wool.Green.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:cyan_wool": {
        lang: "tile.wool.Cyan.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:light_blue_wool": {
        lang: "tile.wool.LightBlue.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:blue_wool": {
        lang: "tile.wool.Blue.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:purple_wool": {
        lang: "tile.wool.Purple.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:magenta_wool": {
        lang: "tile.wool.Magenta.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:pink_wool": {
        lang: "tile.wool.Pink.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    //! Carpet
    "minecraft:white_carpet": {
        lang: "tile.carpet.white.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:light_gray_carpet": {
        lang: "tile.carpet.Silver.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:gray_carpet": {
        lang: "tile.carpet.Gray.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:black_carpet": {
        lang: "tile.carpet.Black.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:brown_carpet": {
        lang: "tile.carpet.Brown.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:red_carpet": {
        lang: "tile.carpet.Red.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:orange_carpet": {
        lang: "tile.carpet.Orange.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:yellow_carpet": {
        lang: "tile.carpet.Yellow.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:lime_carpet": {
        lang: "tile.carpet.Lime.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:green_carpet": {
        lang: "tile.carpet.Green.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:cyan_carpet": {
        lang: "tile.carpet.Cyan.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:light_blue_carpet": {
        lang: "tile.carpet.LightBlue.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:blue_carpet": {
        lang: "tile.carpet.Blue.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:purple_carpet": {
        lang: "tile.carpet.Purple.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:magenta_carpet": {
        lang: "tile.carpet.Magenta.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
    "minecraft:pink_carpet": {
        lang: "tile.carpet.Pink.name",
        tool: {
            type: "shears",
            level: undefined
        }
    },
};
export function getToolIcon(block) {
    const tags = block.getTags();
    const tagToToolMap = {
        "minecraft:is_mace_item_destructible": "mace",
        "minecraft:is_shovel_item_destructible": "shovel",
        "minecraft:is_shears_item_destructible": "shears",
        "minecraft:is_sword_item_destructible": "sword",
        "minecraft:is_tool_item_destructible": "generic_tool",
        "minecraft:is_axe_item_destructible": "axe",
        "minecraft:is_digger_item_destructible": "digger",
        "minecraft:is_pickaxe_item_destructible": "pickaxe",
        "minecraft:is_hoe_item_destructible": "hoe"
    };
    const tagToLevelMap = {
        "minecraft:wood_tier_destructible": 1,
        "minecraft:stone_tier_destructible": 2,
        "minecraft:iron_tier_destructible": 3,
        "minecraft:diamond_tier_destructible": 4,
        "minecraft:netherite_tier_destructible": 5
    };
    const iconMap = {
        sword: { 1: "", 2: "", 3: "", 4: "", 5: "" },
        pickaxe: { 1: "", 2: "", 3: "", 4: "", 5: "" },
        axe: { 1: "", 2: "", 3: "", 4: "", 5: "" },
        shovel: { 1: "", 2: "", 3: "", 4: "", 5: "" },
        hoe: { 1: "", 2: "", 3: "", 4: "", 5: "" },
        shears: ""
    };
    const findMatch = (tags, map) => tags.find(tag => map[tag]);
    let tool = tagToToolMap[findMatch(tags, tagToToolMap)];
    let toolLevel = tagToLevelMap[findMatch(tags, tagToLevelMap)];
    if (tool && toolLevel === undefined) {
        toolLevel = 1;
    }
    const getIcon = (tool, level) => {
        if (tool === "shears")
            return iconMap.shears;
        return iconMap[tool]?.[level];
    };
    let icon = getIcon(tool, toolLevel);
    if (!icon) {
        const data = blocks[block.typeId]?.tool;
        tool = data?.type;
        toolLevel = data?.level;
        icon = getIcon(tool, toolLevel);
    }
    return icon;
}
